import { DepGraph } from 'dependency-graph';
import gqlTag from 'graphql-tag';
export const propsType = convert => ({ name, operationType }, options) => {
    const { noNamespaces } = options.data.root.config || { noNamespaces: false };
    if (operationType === 'mutation') {
        return `
            Partial<
                ReactApollo.MutateProps<
                                        ${noNamespaces ? convert(name, 'typeNames') : ''}Mutation, 
                                        ${noNamespaces ? convert(name, 'typeNames') : ''}Variables
                                        >
                >
        `;
    }
    else {
        return `
            Partial<
                ReactApollo.DataProps<
                                        ${noNamespaces ? convert(name, 'typeNames') : ''}${convert(operationType)}, 
                                        ${noNamespaces ? convert(name, 'typeNames') : ''}Variables
                                    >
                    >
        `;
    }
};
export const generateFragments = convert => (fragments, options) => {
    const cachedFragments = {};
    if (!fragments) {
        return '';
    }
    const graph = new DepGraph({ circular: true });
    fragments.forEach(fragment => {
        graph.addNode(fragment.name, fragment);
    });
    fragments.forEach(fragment => {
        const depends = extractFragments(fragment.document);
        if (depends) {
            depends.forEach(name => {
                graph.addDependency(fragment.name, name);
            });
        }
    });
    return graph
        .overallOrder()
        .map(name => generateFragment(graph.getNodeData(name), options))
        .join('\n');
    function generateFragment(fragment, options) {
        const cached = cachedFragments[fragment.name];
        if (!cached) {
            cachedFragments[fragment.name] = fragment;
            const config = options.data.root.config || {};
            const pascalCasedFragmentName = convert(fragment.name, 'typeNames');
            // fooBar, FooBar and foo_bar may cause conflict due to the pascalCase.
            // Because all of them will have same namespace FooBar
            if (config.noNamespaces) {
                return `
          export const ${pascalCasedFragmentName}FragmentDoc = ${gql(convert)(fragment, options)};
        `;
            }
            else {
                return `
          export namespace ${pascalCasedFragmentName} {
            export const FragmentDoc = ${gql(convert)(fragment, options)};
          }
        `;
            }
        }
        else {
            if (fragment.document !== cached.document) {
                throw new Error(`Duplicated fragment called '${fragment.name}'`);
            }
        }
    }
};
export const gql = convert => (operation, options) => {
    const config = options.data.root.config || {};
    const doc = `
    ${operation.document}
    ${includeFragments(transformFragments(convert)(operation.document, options))}
  `;
    return config.noGraphqlTag ? JSON.stringify(gqlTag(doc)) : 'gql`' + doc + '`';
};
function includeFragments(fragments) {
    if (fragments) {
        return `
      ${fragments
            .filter((name, i, all) => all.indexOf(name) === i)
            .map(name => '${' + name + '}')
            .join('\n')}
    `;
    }
    return '';
}
export function extractFragments(document) {
    return (document.match(/\.\.\.[a-z0-9\_]+/gi) || []).map(name => name.replace('...', ''));
}
const transformFragments = convert => (document, options) => {
    return extractFragments(document).map(document => toFragmentName(convert)(document, options));
};
export const toFragmentName = convert => (fragmentName, options) => {
    const config = options.data.root.config || {};
    if (config.noNamespaces) {
        return convert(`${fragmentName}FragmentDoc`, 'typeNames');
    }
    else {
        return convert(fragmentName, 'typeNames') + '.FragmentDoc';
    }
};
export const shouldOutputHook = (operationType, options) => {
    const config = options.data.root.config || {};
    return operationType !== 'subscription' || config.withSubscriptionHooks;
};
export const gqlImport = (operationType, options) => {
    const config = options.data.root.config || {};
    return config.gqlImport || 'import gql from \'graphql-tag\'';
};
//# sourceMappingURL=helpers.js.map