import { initCommonTemplate } from 'graphql-codegen-typescript-common';
import { transformDocumentsFiles } from 'graphql-codegen-core';
import { flattenTypes } from 'graphql-codegen-plugin-helpers';
import * as Handlebars from 'handlebars';
import * as rootTemplate from './root.handlebars';
import { generateFragments, gql, propsType, shouldOutputHook, gqlImport } from './helpers';
import { extname } from 'path';
export const plugin = async (schema, documents, config) => {
    const { templateContext, convert } = initCommonTemplate(Handlebars, schema, config);
    const transformedDocuments = transformDocumentsFiles(schema, documents);
    const flattenDocuments = flattenTypes(transformedDocuments);
    Handlebars.registerHelper('generateFragments', generateFragments(convert));
    Handlebars.registerHelper('gql', gql(convert));
    Handlebars.registerHelper('propsType', propsType(convert));
    Handlebars.registerHelper('shouldOutputHook', shouldOutputHook);
    Handlebars.registerHelper('gqlImport', gqlImport);
    const hbsContext = Object.assign({}, templateContext, flattenDocuments);
    return Handlebars.compile(rootTemplate)(hbsContext);
};
export const validate = async (schema, documents, config, outputFile) => {
    if (extname(outputFile) !== '.tsx') {
        throw new Error(`Plugin "react-apollo" requires extension to be ".tsx"!`);
    }
};
//# sourceMappingURL=index.js.map